import socket
from . import communicator


class ethernet(communicator.communicator):
    method = 'ethernet'
    
    host = ''
    port = 0
    timeout = 3
    family = ''
    type = ''
    
    def __init__(self, host, port, timeout=3,
                 family=socket.AF_INET,
                 type=socket.SOCK_STREAM):
        self.host = host
        self.port = port
        self.timeout = timeout
        self.family = family
        self.type = type
        pass
    
    def open(self):
        #print('open', self.host, self.port) 
        if self.connection == False:
            self.sock = socket.socket(self.family, self.type)
            self.sock.settimeout(self.timeout)
            self.sock.connect((self.host, self.port))
            self.sockfp = self.sock.makefile()
            self.connection = True
            pass
        return

    def close(self):
        self.sock.close()
        del(self.sock)
        self.connection = False
        return
        
    def send(self, msg):
        self.sock.send((msg+self.terminator).encode())
        return
        
    def recv(self, byte=1024):
        ret = self.sock.recv(byte)
        return ret
    
    def readline(self):
        ret = self.sockfp.readline()
        return ret






# Signal Generator
# ----------------
from .E8257 import E8257D
from .E8247 import E8247C

# Power Meter
# -----------
from .E4418 import EPM441A
from .E4418 import EPM442A
from .E4418 import E4418B
from .E4418 import E4419B

# Spectrum Analyzer
# -----------------
from .N9342 import N9342C
from .N9342 import N9343C
from .N9342 import N9344C

# Attenuator/Switch Driver
# ------------------------
from .agilent_11713 import agilent_11713B
from .agilent_11713 import agilent_11713C




from ..SCPI import scpi

# helper classes
# ==============

# Channel List
# ------------
class channel_list(object):
    ch_available = []
    
    def __init__(self, ch_list):
        target_ch = []
        if type(ch_list) == str:
            ch_list = ch_list.strip('(@)')
            if ch_list.find(':') != -1:
                v1, v2 = ch_list.split(':')
                target_ch = range(int(v1), int(v2)+1)
            elif ch_list.find(',') != -1:
                v = ch_list.split(',')
                target_ch = [int(_v) for _v in v]
            else:
                target_ch = [int(ch_list)]
                pass
            pass
        elif type(ch_list) in [list, tuple]:
            target_ch = ch_list
        else:
            target_ch = [ch_list]
            pass
        self.ch = target_ch
        self._verify()
        self._make_query()
        pass
        
    def _verify(self):
        for c in self.ch:
            if not c in self.ch_available:
                raise ValueError('ch %d is not supported.'%(c))
            continue
        return
        
    def _make_query(self):
        self.query = '(@' + ','.join(['%03d'%c for c in self.ch]) + ')' 
        return

class channel_list_11713B(channel_list):
    ch_available = [101, 102, 103, 104, 105, 106, 107, 108, 109, 110]

class channel_list_11713C(channel_list):
    ch_available = [101, 102, 103, 104, 105, 106, 107, 108, 109, 110,
                    201, 202, 203, 204, 205, 206, 207, 208, 209, 210]
    
# Supply Voltage
# --------------
class switch_voltage(object):
    available = {'OFF': 0, 'P5v': 5, 'P15v': 15, 'P24v': 24, 'USER': None}
    
    def __init__(self, val):
        if type(val) == int:
            if not val in self.available.values():
                raise ValueError('voltage %d is not supported.'%(val))
            self.int = val
            self.str = [key for key, value in self.available.items() if value==val][0]
        elif type(val) == str:
            if val[0]=='P' & val[-1]!='v': val += 'v'
            if not val in self.available.keys():
                raise ValueError('voltage %s is not supported.'%(val))
            self.str = val
            self.int = self.available[val]
        else:
            raise TypeError('voltage should be specified as int or str')
        pass
            
# Bank Number
# -----------
class bank_number(object):
    available = [1, 2]
    
    def __init__(self, val):
        if type(val) == int:
            if not val in self.available:
                raise ValueError('voltage %d is not supported.'%(val))
            self.int = val
        else:
            raise TypeError('voltage should be specified as int or str')
        pass
            
# ON - OFF
# --------
class on_off(object):
    available = {'OFF': 0, 'ON': 1}
    
    def __init__(self, val):
        if type(val) in [int, bool]:
            val = int(val)
            if not val in self.available.values():
                raise ValueError('voltage %d is not supported.'%(val))
            self.int = val
            self.str = [key for key, value in self.available.items() if value==val][0]
        elif type(val) == str:
            if val[0]=='P' & val[-1]!='v': val += 'v'
            if not val in self.available.keys():
                raise ValueError('voltage %s is not supported.'%(val))
            self.str = val
            self.int = self.available[val]
        else:
            raise TypeError('voltage should be specified as int or str')
        pass
            



# main class
# ==========

class agilent_11713(scpi.scpi_family):
    manufacturer = 'Agilent'
    product_name = '11713'
    classification = 'Attenuator/Switch Driver'
    
    _scpi_enable = '*IDN?'
    
    _ch_list = channel_list
    
    def switch_open(self, ch):
        """
        :ROUTe:OPEn : Open ch(s)
        ------------------------
        Open switching path(s).
        
        Args
        ====
        < ch : ch_list  :  >
            Specify ch(s) to open.
            <ch> should be str, list of int, or int.
            
            str : Specify the ch(s) by characters.
                  Following formats are available.
                   - single ch : '101'
                   - multiple chs : '102, 104, 108'
                   - a range of chs : '101:105'
        
            list : Specify the multiple chs by a list of int.
                   e.g., [101, 102, 105]
          
            int : Specify the ch by a int value.
                  e.g., 106
        
        Returns
        =======
        Nothing.
        
        Examples
        ========
        >>> a.switch_open('101')
        >>> a.switch_open('101, 102')
        >>> a.switch_open('101:104')
        >>> a.switch_open([101, 102])
        >>> a.switch_open(101)
        """
        ch = self._ch_list(ch)
        self.com.send(':ROUTe:OPEn %s'%(ch.query))
        return
        
    def switch_close(self, ch):
        """
        :ROUTe:CLOSe : Close ch(s)
        --------------------------
        Close switching path(s).
        
        Args
        ====
        < ch : ch_list  :  >
            Specify ch(s) to close.
            <ch> should be str, list of int, or int.
            
            str : Specify the ch(s) by characters.
                  Following formats are available.
                   - single ch : '101'
                   - multiple chs : '102, 104, 108'
                   - a range of chs : '101:105'
        
            list : Specify the multiple chs by a list of int.
                   e.g., [101, 102, 105]
          
            int : Specify the ch by a int value.
                  e.g., 106
        
        Returns
        =======
        Nothing.
        
        Examples
        ========
        >>> a.switch_close('101')
        >>> a.switch_close('101, 102')
        >>> a.switch_close('101:104')
        >>> a.switch_close([101, 102])
        >>> a.switch_close(101)
        """
        ch = self._ch_list(ch)
        self.com.send(':ROUTe:CLOSe %s'%(ch.query))
        return

    def switch_open_all(self):
        """
        :ROUTe:OPEn:ALL : Open all chs
        ------------------------------
        Open all switching paths.
        
        Args
        ====
        Nothing
        
        Returns
        =======
        Nothing.
        
        Examples
        ========
        >>> a.switch_open_all()
        """
        self.com.send(':ROUTe:OPEn:ALL')
        return
        
    def switch_close_all(self):
        """
        :ROUTe:CLOSe:ALL : Close all chs
        --------------------------------
        Close all switching paths.
        
        Args
        ====
        Nothing
        
        Returns
        =======
        Nothing.
        
        Examples
        ========
        >>> a.switch_close_all()
        """
        self.com.send(':ROUTe:CLOSe:ALL')
        return
        
    def switch_open_query(self, ch):
        """
        :ROUTe:OPEn? : Query open ch(s)
        -------------------------------
        Query switching path is open.
        
        Args
        ====
        < ch : ch_list  :  >
            Specify ch(s) to check open.
            <ch> should be str, list of int, or int.
            
            str : Specify the ch(s) by characters.
                  Following formats are available.
                   - single ch : '101'
                   - multiple chs : '102, 104, 108'
                   - a range of chs : '101:105'
        
            list : Specify the multiple chs by a list of int.
                   e.g., [101, 102, 105]
          
            int : Specify the ch by a int value.
                  e.g., 106
        
        Returns
        =======
        < is_open : list(int): >
            Return a list of open_flag for each chs requested.
            0 = close, 1 = open
        
        Examples
        ========
        >>> a.switch_open_query('101')
        [1]
        
        >>> a.switch_open_query('101, 102')
        [1, 0]
        
        >>> a.switch_open_query('101:104')
        [1, 0, 1, 0]
        
        >>> a.switch_open_query([101, 102])
        [1, 0]
        
        >>> a.switch_open_query(101)
        [1]
        """
        ch = self._ch_list(ch)
        self.com.send(':ROUTe:OPEn? %s'%(ch.query))
        ret = self.com.readline()
        ret = [int(r) for r in ret.strip().split(',')]
        return ret

    def switch_close_query(self, ch):
        """
        :ROUTe:CLOSe? : Query close ch(s)
        ---------------------------------
        Query switching path is close.
        
        Args
        ====
        < ch : ch_list  :  >
            Specify ch(s) to check close.
            <ch> should be str, list of int, or int.
            
            str : Specify the ch(s) by characters.
                  Following formats are available.
                   - single ch : '101'
                   - multiple chs : '102, 104, 108'
                   - a range of chs : '101:105'
        
            list : Specify the multiple chs by a list of int.
                   e.g., [101, 102, 105]
          
            int : Specify the ch by a int value.
                  e.g., 106
        
        Returns
        =======
        < is_close : list(int): >
            Return a list of close_flag for each chs requested.
            0 = open, 1 = close
        
        Examples
        ========
        >>> a.switch_close_query('101')
        [0]
        
        >>> a.switch_close_query('101, 102')
        [0, 1]
        
        >>> a.switch_close_query('101:104')
        [0, 1, 0, 1]
        
        >>> a.switch_close_query([101, 102])
        [0, 1]
        
        >>> a.switch_close_query(101)
        [0]
        """
        ch = self._ch_list(ch)
        self.com.send(':ROUTe:CLOSe? %s'%(ch.query))
        ret = self.com.readline()
        ret = [int(r) for r in ret.strip().split(',')]
        return ret
        
    def supply_voltage_set(self, voltage, bank=1):
        """
        :CONFigure:BANKn : Set Supply Voltage
        -------------------------------------
        Set the supply voltage for specified bank.
        
        Args
        ====
        < voltage : switch_voltage  :  >
            Specify the voltage to apply.
            <voltage> should be str, or int.
            
            str : Specify the voltage by characters.
                  voltage = 'OFF', 'P5v', 'P15v', 'P24v', or 'USER'
            
            int : Specify the voltage by a int value.
                  0 = OFF,  5 = P5v,  15 = P15v,  24 = P24v
        
        < bank : int : 1,2 >
            Specify the bank to apply the voltage.
            bank = 1 or 2. default is bank = 1.
        
        Returns
        =======
        Nothing.
        
        Examples
        ========
        >>> a.switch_supply_voltage_set('P24v')
        >>> a.switch_supply_voltage_set(5)
        >>> a.switch_supply_voltage_set('P24v', bank=2)
        """
        voltage = switch_voltage(voltage)
        bank = bank_number(bank)
        self.com.send('CONFigure:BANK%d %s'%(bank.int, voltage.str))
        return
    
    def supply_voltage_query(self, bank=1):
        """
        :CONFigure:BANKn? : Query Supply Voltage
        -----------------------------------------
        Query the supply voltage for specified bank.
        
        Args
        ====
        < bank : int : 1,2 >
            Specify the bank to check the voltage.
            bank = 1 or 2. default is bank = 1.
        
        Returns
        =======
        < voltage : str : 'OFF', 'P5', 'P15', 'P24', 'USER' >
            Return voltage which is applyed to the switch.
            <voltage> is 'OFF', 'P5', 'P15', 'P24', or 'USER'.
        
        Examples
        ========
        >>> a.switch_supply_voltage_query()
        'P24'
        
        >>> a.switch_supply_voltage_query(bank=2)
        'P5'
        """
        bank = bank_number(bank)
        self.com.send('CONFigure:BANK%d?'%(bank.int))
        ret = self.com.readline().strip()
        return ret
        
    def switch_ttl_on_off(self, ttl, bank=1):
        """
        :CONFigure:BANKn:TTL : Set TTL ON/OFF
        -------------------------------------
        Set TTL ON/OFF for specified bank.
        
        Args
        ====
        < ttl : on_off  : 0,1,'ON,'OFF',True,False  >
            Specify the TTL ON/OFF.
            <ttl> should be str, int, or bool.
            
            str : ttl = 'ON', or 'OFF'            
            int : 1 = 'ON', or 0 = 'OFF'
            bool: True = 'ON', or False = 'OFF'

        < bank : int : 1,2 >
            Specify the bank to configure TTL.
            bank = 1 or 2. default is bank = 1.
        
        Returns
        =======
        Nothing.
        
        Examples
        ========
        >>> a.switch_ttl_on_off('ON')
        >>> a.switch_ttl_on_off(0, bank=2)
        """
        ttl = on_off(ttl)
        bank = bank_number(bank)
        self.com.send('CONFigure:BANK%d:TTL %s'%(bank.int, ttl.str))
        return
        
    def switch_ttl_on_off_query(self, bank=1):
        """
        :CONFigure:BANKn:TTL? : Query TTL ON/OFF
        ----------------------------------------
        Query TTL ON/OFF for specified bank.
        
        Args
        ====
        < bank : int : 1,2 >
            Specify the bank to configure TTL.
            bank = 1 or 2. default is bank = 1.
        
        Returns
        =======
        < ttl : int  : 0,1 >
            Return TTL ON/OFF.
            0 = OFF,  1 = ON
        
        Examples
        ========
        >>> a.switch_ttl_on_off_query()
        0
        
        >>> a.switch_ttl_on_off_query(bank=2)
        1
        """
        bank = bank_number(bank)
        self.com.send('CONFigure:BANK%d:TTL?'%(bank.int))
        ret = self.com.readline()
        ret = int(ret)
        return ret
        
    def relay_cycles_query(self, ch):
        """
        :DIAGnostic:RELay:CYCles? : Query Number of Relay Cycles
        --------------------------------------------------------
        Query the number of relay cycles for each channel.
        
        Args
        ====
        < ch : ch_list  :  >
            Specify ch(s) to check the number of relay cycles.
            <ch> should be str, list of int, or int.
            
            str : Specify the ch(s) by characters.
                  Following formats are available.
                   - single ch : '101'
                   - multiple chs : '102, 104, 108'
                   - a range of chs : '101:105'
        
            list : Specify the multiple chs by a list of int.
                   e.g., [101, 102, 105]
          
            int : Specify the ch by a int value.
                  e.g., 106
        
        Returns
        =======
        < number_of_cycles : list(int): >
            Return a list of number of cycles for each ch.
        
        Examples
        ========
        >>> a.relay_cycles_query('101')
        [123]
        
        >>> a.relay_cycles_query('101, 102')
        [123, 103]
        
        >>> a.relay_cycles_query('101:104')
        [123, 103, 43, 28]
        
        >>> a.relay_cycles_query([101, 102])
        [123, 103]
        
        >>> a.relay_cycles_query(101)
        [123]
        """
        ch = self._ch_list(ch)
        self.com.send(':DIAGnostic:RELay:CYCles? %s'%(ch.query))
        ret = self.com.readline()
        ret = [int(r) for r in ret.strip().split(',')]
        return ret
        
    def relay_cycles_clear(self, ch):
        """
        :DIAGnostic:RELay:CLEAr : Clear Relay Cycles
        --------------------------------------------
        Clear the number of relay cycles for each channel.
        
        Args
        ====
        < ch : ch_list  :  >
            Specify ch(s) to clear the number of relay cycles.
            <ch> should be str, list of int, or int.
            
            str : Specify the ch(s) by characters.
                  Following formats are available.
                   - single ch : '101'
                   - multiple chs : '102, 104, 108'
                   - a range of chs : '101:105'
        
            list : Specify the multiple chs by a list of int.
                   e.g., [101, 102, 105]
          
            int : Specify the ch by a int value.
                  e.g., 106
        
        Returns
        =======
        Nothing
        
        Examples
        ========
        >>> a.relay_cycles_clear('101')
        >>> a.relay_cycles_clear('101, 102')
        >>> a.relay_cycles_clear('101:104')
        >>> a.relay_cycles_clear([101, 102])
        >>> a.relay_cycles_clear(101)
        """
        ch = self._ch_list(ch)
        self.com.send(':DIAGnostic:RELay:CLEAr %s'%(ch.query))
        return 
        
    
        
    _shortcut_command = {'OPEn': 'switch_open',
                         'CLOSe': 'switch_close',
                         'OPEnALL': 'switch_open_all',
                         'CLOSeALL': 'switch_close_all',
                         'OPEnQ': 'switch_open_query',
                         'CLOSeQ': 'switch_close_query',  
                         'BANK': 'supply_voltage_set', 
                         'BANKQ': 'supply_voltage_query',    
                         'TTL': 'switch_ttl_on_off',
                         'TTLQ': 'switch_ttl_on_off_query',
                         'CYClesQ': 'relay_cycles_query',
                         'RELayCLEAr': 'relay_cycles_clear'}


class agilent_11713B(agilent_11713):
    product_name = '11713B'
    _ch_list = channel_list_11713B
    
class agilent_11713C(agilent_11713):
    product_name = '11713C'
    _ch_list = channel_list_11713C
    



import serial
from serial.tools import list_ports
from . import communicator

class usb(communicator.communicator):
    method = 'usb'

    serial_number = ''
    port = ''
    rtimeout = 0.1
    wtimeout = 0.1

    def __init__(self, serial_number):
        self.serial_number = serial_number.upper()
        #self.rtimeout = timeout
        #self.wtimeout = timeout
        serial_li = list_ports.comports()
        for i in range(len(serial_li)):
            try:
                if serial_li[i].serial_number == self.serial_number:
                    self.port = serial_li[i].device
                else: pass
            except: pass
        return

    def open(self):
        if self.connection == False:
            self.ser = serial.Serial(
                self.port, timeout=self.rtimeout, write_timeout=self.wtimeout
            )
            self.connection = True
        return

    def close(self):
        if self.connection == True:
            self.ser.close()
            self.connection = False
        return

    def send(self, msg):
        self.ser.write(msg)
        return

    def recv(self, byte=1024):
        ret = self.ser.read(byte)
        return ret




import pymeasure

com = pymeasure.gpib_prologix('192.168.40.33', 30)
m = pymeasure.Lakeshore.model218(com)

all_ch = range(1, 9)


# alarm_query
# -----------
for i in all_ch:
    m.alarm_query(i)
    continue

m.alarm_query_all()

# alarm_set
# ---------
a = m.alarm_query(8)
m.alarm_set(8, 1, 1, 300, 100, 1, 0)
m.alarm_query(8)
m.alarm_set(8, *a)
m.alarm_query(8)

# alarm_status_query
# ------------------
for i in all_ch:
    m.alarm_status_query(1)
    continue

m.alarm_status_query_all()

# audible_alarm_query
# -------------------
m.audible_alarm_query()

# audible_alarm_set
# -----------------
m.audible_alarm_set(1)
m.audible_alarm_query()
m.audible_alarm_set(0)
m.audible_alarm_query()

# alarm_reset
# -----------
m.alarm_reset()

# analog_outputs_query
# --------------------
m.analog_outputs_query(1)
m.analog_outputs_query(2)

# analog_outputs_set
# ------------------
a = m.analog_outputs_query(1)
m.analog_outputs_set(1, 0, 1, 1, 1, 300, 3)
m.analog_outputs_query(1)

for i in [1,2,3,4,5]:
    m.analog_outputs_set(1, 0, mode=2, manual=i)
    m.analog_outputs_query(1)
    continue

m.analog_outputs_set(1, *a)
m.analog_outputs_query(1)

# analog_output_data_query
# ------------------------
m.analog_output_data_query(1)
m.analog_output_data_query(2)

# serial_interface_baud_rate_query
# --------------------------------
m.serial_interface_baud_rate_query()

# serial_interface_baud_rate_set
# ------------------------------
a = m.serial_interface_baud_rate_query()
m.serial_interface_baud_rate_set(1)
m.serial_interface_baud_rate_query()
m.serial_interface_baud_rate_set(2)
m.serial_interface_baud_rate_query()
m.serial_interface_baud_rate_set(a)
m.serial_interface_baud_rate_query()

# celsius_reading_query
# ---------------------
m.celsius_reading_query(0)
m.celsius_reading_query(1)
m.celsius_reading_query(2)

# curve_header_query
# ------------------
m.curve_header_query(21)
m.curve_header_query(22)
m.curve_header_query(23)

m.curve_header_query_all()

# curve_header_set
# ----------------
a = m.curve_header_query(28)
m.curve_header_set(28, 'TEST', 'TEST001', 2, 350, 1)
m.curve_header_query(28)
m.curve_header_set(28, *a)
m.curve_header_query(28)

# curve_point_query
# -----------------
m.curve_point_query(1, 1)
m.curve_point_query(1, 2)
m.curve_point_query(1, 3)

c1 = m.curve_point_query_line(1)
c2 = m.curve_point_query_line(2)

# import pylab
# pylab.plot(c1[0], c1[1])
# pylab.show()

# curve_point_set
# ---------------
c28 = m.curve_point_query_line(28)
m.curve_point_set(28, 1, 0, 10)
m.curve_point_query(28, 1)
m.curve_point_set(28, 2, 2, 15)
m.curve_point_query(28, 2)

m.curve_point_set_line(28, c1[0], c1[1])
c28_2 = m.curve_point_query_line(28)
m.curve_point_set_line(28, c28[0], c28[1])
c28_3 = m.curve_point_query_line(28)

# datetime_query
# --------------
m.datetime_query()

# datetime_set
# ------------
m.datetime_set(13, 12, 18, 17, 12, 0)
m.datetime_query()

m.datetime_set_now()
m.datetime_query()

# factory_defaults_reset
# ----------------------
m.factory_defaults_reset()

# display_field_query
# -------------------
m.display_field_query(1)
m.display_field_query(2)

m.display_field_query_all()

# display_field_set
# -----------------
a = m.display_field_query(8)
m.display_field_set(8, 1, 3)
m.display_field_query(8)
m.display_field_set(8, *a)
m.display_field_query(8)

m.display_field_set_all_kelvin()

# filter_query
# ------------
m.filter_query(1)
m.filter_query(2)
m.filter_query(3)

m.filter_query_all()

# filter_set
# ----------
a = m.filter_query(8)
m.filter_set(8, 1, 5, 2)
m.filter_query(8)
m.filter_set(8, *a)
m.filter_query(8)

# ieee488_query
# -------------
m.ieee488_query()

# ieee488_set
# -----------
a = m.ieee488_query()
m.ieee488_set()
m.ieee488_query()

# input_curve_query
# -----------------
m.input_curve_query(1)
m.input_curve_query(2)
m.input_curve_query(3)

m.input_curve_query_all()

# input_curve_set
# ---------------
a = m.input_curve_query(8)
m.input_curve_set(8, 28)
m.input_curve_query(8)
m.input_curve_set(8, 27)
m.input_curve_query(8)
m.input_curve_set(8, a)
m.input_curve_query(8)

# input_control_query
# -------------------
m.input_control_query(1)
m.input_control_query(2)
m.input_control_query(3)

m.input_control_query_all()

# input_control_set
# -----------------
a = m.input_control_query(8)
m.input_control_set(8, 1)
m.input_control_query(8)
m.input_control_set(8, 0)
m.input_control_query(8)
m.input_control_set(8, a)
m.input_control_query(8)

# input_type_query
# ----------------
m.input_type_query('A')
m.input_type_query('B')

m.input_type_query_all()

# input_type_set
# --------------
a = m.input_type_query('B')
m.input_type_set('B', 2)
m.input_type_query('B')
m.input_type_set('B', 3)
m.input_type_query('B')
m.input_type_set('B', a)
m.input_type_query('B')

# keypad_status_query
# -------------------
m.keypad_status_query()

# kelvin_reading_query
# --------------------
m.kelvin_reading_query(0)
m.kelvin_reading_query(1)
m.kelvin_reading_query(2)

# linear_equation_query
# ---------------------
m.linear_equation_query(1)
m.linear_equation_query(2)
m.linear_equation_query(3)

m.linear_equation_query_all()

# linear_equation_set
# -------------------
a = m.linear_equation_query(8)
m.linear_equation_set(8, 1, 1, 5)
m.linear_equation_query(8)
m.linear_equation_set(8, *a)
m.linear_equation_query(8)

# lockout_query
# -------------
m.lockout_query()

# lockout_set
# -----------
m.lockout_set(1, 123)
m.lockout_query()
m.lockout_set(0, 123)
m.lockout_query()

# logging_on_off_query
# --------------------
m.logging_on_off_query()

# logging_on_off
# --------------
m.logging_on_off(1)
m.logging_on_off_query()
m.logging_on_off(0)
m.logging_on_off_query()

# logging_number_query
# --------------------
m.logging_number_query()

# logging_records_query
# ---------------------
m.logging_records_query(1)
m.logging_records_query(2)
m.logging_records_query(3)

m.logging_records_query_all()

# logging_records_set
# -------------------
a = m.logging_records_query(7)
m.logging_records_set(7, 8, 1)
m.logging_records_query(7)
m.logging_records_set(7, *a)
m.logging_records_query(7)

# logging_parameter_query
# -----------------------
m.logging_parameter_query()

# logging_parameter_set
# ---------------------
a = m.logging_parameter_query()
m.logging_parameter_set(0, 1, 0, 10, 1)
m.logging_parameter_query()
m.logging_parameter_set(*a)
m.logging_parameter_query()

# log_data_query
# --------------
m.log_data_query(1, 1)

# linear_equation_input_data_query
# --------------------------------
m.linear_equation_input_data_query(0)
m.linear_equation_input_data_query(1)
m.linear_equation_input_data_query(2)

# minmax_query
# ------------
m.minmax_query(1)
m.minmax_query(2)
m.minmax_query(3)

# minmax_set
# ----------
m.minmax_set(8, 1)

# minmax_data_query
# -----------------
m.minmax_data_query(1)
m.minmax_data_query(2)
m.minmax_data_query(3)

# minmax_function_reset
# ---------------------
m.minmax_function_reset()

# local_remote_mode_query
# -----------------------
m.local_remote_mode_query()

# local_remote_mode_set
# ---------------------
m.local_remote_mode_set(0)
m.local_remote_mode_query()
m.local_remote_mode_set(1)
m.local_remote_mode_query()
m.local_remote_mode_set(2)
m.local_remote_mode_query()
m.local_remote_mode_set(1)
m.local_remote_mode_query()

# reading_status_query
# --------------------
m.reading_status_query(1)
m.reading_status_query(2)
m.reading_status_query(3)

# relay_query
# -----------
m.relay_query(1)
m.relay_query(2)
m.relay_query(3)

# relay_set
# ---------
m.relay_set(1, 0, 1, 0)
m.relay_query(1)

# relay_status_query
# ------------------
m.relay_status_query()

# softcal_curve_generate
# ----------------------

# sensor_units_reading_query
# --------------------------
m.sensor_units_reading_query(0)
m.sensor_units_reading_query(1)
m.sensor_units_reading_query(2)




from .SCPI import scpi_common

from . import Agilent
from . import Agilent as Keysight
from . import Phasematrix
from . import Lakeshore
from . import ELVA1
from . import Pfeiffer
from . import Anritsu